package com.example.app.jmix.playground.playground.screen.login;

import java.util.Locale;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.LockedException;

import com.example.app.jmix.playground.playground.JmixPlaygroundApplication;

import io.jmix.core.MessageTools;
import io.jmix.core.Messages;
import io.jmix.securityui.authentication.AuthDetails;
import io.jmix.securityui.authentication.LoginScreenSupport;
import io.jmix.ui.JmixApp;
import io.jmix.ui.Notifications;
import io.jmix.ui.action.Action;
import io.jmix.ui.component.CheckBox;
import io.jmix.ui.component.ComboBox;
import io.jmix.ui.component.HasValue;
import io.jmix.ui.component.PasswordField;
import io.jmix.ui.component.TextField;
import io.jmix.ui.navigation.Route;
import io.jmix.ui.screen.OpenMode;
import io.jmix.ui.screen.Screen;
import io.jmix.ui.screen.Subscribe;
import io.jmix.ui.screen.UiController;
import io.jmix.ui.screen.UiControllerUtils;
import io.jmix.ui.screen.UiDescriptor;
import io.jmix.ui.security.UiLoginProperties;

@UiController(LoginScreen.UI_CONTROLLER)
@UiDescriptor(LoginScreen.UI_DESCRIPTOR)
@Route(path = LoginScreen.ROUTE_PATH, root = true)
public class LoginScreen extends Screen
{
    /**
     * Defines the navigation route path.
     */
    public static final String ROUTE_PATH = "login";

    /**
     * Defines the UI controller name.
     */
    public static final String UI_CONTROLLER = JmixPlaygroundApplication.SCREEN_PREFIX + "LoginScreen";

    /**
     * Defines the UI descriptor file name.
     */
    public static final String UI_DESCRIPTOR = "login-screen.xml";

    @Autowired
    private Notifications notifications;

    @Autowired
    private Messages messages;

    @Autowired
    private MessageTools messageTools;

    @Autowired
    private LoginScreenSupport loginScreenSupport;

    @Autowired
    private JmixApp app;

    @Autowired
    private UiLoginProperties loginProperties;

    @Autowired
    private TextField<String> usernameField;

    @Autowired
    private PasswordField passwordField;

    @Autowired
    private CheckBox rememberMeCheckBox;

    @Autowired
    private ComboBox<Locale> localesField;

    private final Logger log = LoggerFactory.getLogger(LoginScreen.class);

    @Subscribe
    private void onInit(final InitEvent anEvent)
    {
        usernameField.focus();
        initLocalesField();
        initDefaultCredentials();
    }

    private void initLocalesField()
    {
        localesField.setOptionsMap(messageTools.getAvailableLocalesMap());
        localesField.setValue(app.getLocale());
        localesField.addValueChangeListener(this::onLocalesFieldValueChangeEvent);
    }

    private void onLocalesFieldValueChangeEvent(final HasValue.ValueChangeEvent<Locale> anEvent)
    {
        //noinspection ConstantConditions
        app.setLocale(anEvent.getValue());
        UiControllerUtils.getScreenContext(this).getScreens().create(getClass(), OpenMode.ROOT).show();
    }

    private void initDefaultCredentials()
    {
        final String defaultUsername = loginProperties.getDefaultUsername();
        if (!StringUtils.isBlank(defaultUsername) && !"<disabled>".equals(defaultUsername))
        {
            usernameField.setValue(defaultUsername);
        }
        else
        {
            usernameField.setValue("");
        }

        final String defaultPassword = loginProperties.getDefaultPassword();
        if (!StringUtils.isBlank(defaultPassword) && !"<disabled>".equals(defaultPassword))
        {
            passwordField.setValue(defaultPassword);
        }
        else
        {
            passwordField.setValue("");
        }
    }

    @Subscribe("submit")
    private void onSubmitActionPerformed(final Action.ActionPerformedEvent anEvent)
    {
        login();
    }

    private void login()
    {
        final String username = usernameField.getValue();
        final String password = passwordField.getValue();

        if (StringUtils.isEmpty(username) || StringUtils.isEmpty(password))
        {
            notifications.create(Notifications.NotificationType.WARNING)
                            .withCaption(messages.getMessage(getClass(), "emptyUsernameOrPassword")).show();
            return;
        }

        try
        {
            loginScreenSupport.authenticate(AuthDetails.of(username, password).withLocale(localesField.getValue())
                            .withRememberMe(rememberMeCheckBox.isChecked()), this);
        }
        catch (final BadCredentialsException | DisabledException | LockedException ex)
        {
            log.info("Login failed", ex);
            notifications.create(Notifications.NotificationType.ERROR)
                            .withCaption(messages.getMessage(getClass(), "loginFailed"))
                            .withDescription(messages.getMessage(getClass(), "badCredentials")).show();
        }
    }
}
